"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _trieCodec = _interopRequireDefault(require("@polkadot/trie-codec"));

var _constants = require("@polkadot/trie-codec/constants");

var _util = require("@polkadot/util");

var _Cache = _interopRequireDefault(require("./Cache"));

var _Impl = _interopRequireDefault(require("./Impl"));

var _constants2 = require("./constants");

var _util2 = require("./util");

// Copyright 2017-2019 @polkadot/client-db authors & contributors
// This software may be modified and distributed under the terms
// of the Apache-2.0 license. See the LICENSE file for details.
const l = (0, _util.logger)('db/struct');
const TRIE_BRANCH_KEYS = 16;
const TRIE_BRANCH_LEN = TRIE_BRANCH_KEYS + 1;
const TRIE_ENC_SIZE = _constants2.U32_SIZE + 2;

class FileStructDb extends _Impl.default {
  constructor() {
    super(...arguments);
    this._cache = new _Cache.default(16 * 1024);
  }

  drop() {
    l.error('drop() is not implemented');
  }

  empty() {
    l.error('empty() is not implemented');
  } // eslint-disable-next-line @typescript-eslint/no-unused-vars


  rename(base, file) {
    l.error('rename() is not implemented');
  }

  size() {
    l.error('size() is not implemented');
    return 0;
  }

  maintain(fn) {
    fn({
      isCompleted: true,
      keys: 0,
      percent: 100
    });
  } // eslint-disable-next-line @typescript-eslint/no-unused-vars


  del(key) {
    l.error('del() is not implemented');
  }

  _get(key) {
    const info = this._findValue((0, _util2.serializeKey)(key)); // console.log('_get', this._isTrie, key, info);


    if (!info || !info.valData) {
      return null;
    } else if (!this._isTrie) {
      return info.valData;
    } else if (!info.valData[0]) {
      return info.valData.subarray(1);
    } // 17 entries, assuming there may be an encoded value in there


    const recoded = [null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null, null];
    let offset = 3;
    const bitmap = info.valData[1] + (info.valData[2] << 8);

    for (let index = 0; index < TRIE_BRANCH_KEYS; index++) {
      if (bitmap & _constants.BITMAP[index]) {
        const fileAt = info.valData[offset++];
        const [length, keyAt] = (0, _util.compactFromU8a)(info.valData.subarray(offset));
        recoded[index] = this._readKey(fileAt, keyAt.toNumber()).subarray(0, _constants2.KEY_DATA_SIZE);
        offset += length;
      }
    }

    return _trieCodec.default.encode(recoded);
  }

  get(key) {
    // l.debug(() => ['get', { key }]);
    const keyStr = key.toString();

    const cached = this._cache.get(keyStr);

    if (cached) {
      return cached;
    }

    const value = this._get(key);

    if (value) {
      this._cache.set(keyStr, value);
    }

    return value;
  }

  put(key, value) {
    // l.debug(() => ['put', { key, value }]);
    this._cache.set(key.toString(), value);

    if (!this._isTrie) {
      this._findValue((0, _util2.serializeKey)(key), value);

      return;
    }

    let adjusted;

    const decoded = _trieCodec.default.decode(value);

    const isEncodable = Array.isArray(decoded) && decoded.length === TRIE_BRANCH_LEN && !decoded.some(value => !!value && value.length !== _constants2.KEY_DATA_SIZE); // extension nodes are going away anyway, so just ignore, no worse off.

    if (isEncodable) {
      const recoded = new Uint8Array(TRIE_BRANCH_KEYS * TRIE_ENC_SIZE + 1 + 2);
      let length = 3;
      let bitmap = 0;

      for (let index = 0; index < TRIE_BRANCH_KEYS; index++) {
        const u8a = decoded[index];

        if (u8a) {
          const key = (0, _util2.serializeKey)(u8a);

          const info = this._findValue(key, null, false);

          const atEnc = (0, _util.compactToU8a)(info.keyAt);
          bitmap |= _constants.BITMAP[index];
          recoded[length++] = key.fileAt;
          recoded.set(atEnc, length);
          length += atEnc.length;
        }
      }

      recoded[0] = TRIE_BRANCH_KEYS;
      recoded[1] = bitmap & 0xff;
      recoded[2] = bitmap >> 8;
      adjusted = recoded.subarray(0, length);
    } else {
      adjusted = new Uint8Array(value.length + 1);
      adjusted.set(value, 1);
    }

    this._findValue((0, _util2.serializeKey)(key), adjusted);
  }

}

exports.default = FileStructDb;